package logic;

import java.io.*;
import java.util.*;

/**
 * The Collection for the books to easily be loaded and saved
 * @author Jens
 *
 */
public class BookCollection implements Serializable{
	private static final long serialVersionUID = 1L;
	private long idcounter;
	private ArrayList<Book> books;
	private static BookCollection instance;
	
	/**
	 * the instance, we need only one book collection
	 * @return the one and only instance
	 */
	public static BookCollection getInstance() {
		if(instance==null){
			instance = new BookCollection();
		}
		return instance;
	}
	
	/**
	 * Sets the instance, for load methods
	 * @param instance the instance to load
	 */
	public static void setInstance(BookCollection instance) {
		BookCollection.instance = instance;
	}
	
	/**
	 * This constructor initializes the counter and the arraylist 
	 */
	public BookCollection(){
		this.idcounter = 0;
		this.books = new ArrayList<Book>();
	}
	
	/**
	 * adds a random book
	 * @return the id of the new book
	 */
	public long addBook(){
		this.books.add(new Book(this.idcounter));
		this.idcounter++;
		return this.idcounter-1;
	}
	
	/**
	 * Removes a book by a given id
	 * @param id the id of the book to delete
	 * @return true, if delete was successful
	 * @throws BookNotFoundException if this book was not found
	 */
	public boolean removeBook(long id) throws BookNotFoundException{
		return this.books.remove(getBook(id));
	}
	
	/**
	 * gets a specific book from the collection by the id, used
	 * by the other methods as well 
	 * @param id the id of the bokk
	 * @return the book
	 * @throws BookNotFoundException if this book was not found
	 */
	public Book getBook(long id) throws BookNotFoundException{
		for(Book book : this.books){
			if(book.getId()==id){
				return book;
			}
		}
		throw new BookNotFoundException();
	}
	
	/**
	 * Adds a given notice to a book
	 * @param id thie id of the book to be noticed
	 * @param notice the notice-string
	 * @throws BookNotFoundException if this book was not found
	 */
	public void addNotice(long id, String notice) throws BookNotFoundException{
		this.getBook(id).addNotice(notice);
	}
	
	/**
	 * Removes a specific notice from a specific book
	 * @param id the id of the book
	 * @param notice the notice-string to be removed
	 * @return true, is remove was successful
	 * @throws BookNotFoundException if this book was not found
	 */
	public boolean removeNotice(long id, String notice) throws BookNotFoundException{
		return this.getBook(id).removeNotice(notice);
	}
	
	/**
	 * Returns the toString-method of each book in the collection,
	 * separated by line wraps. Might be quite a long string...
	 */
	public String toString(){
		String collection = "";
		for(Book book : books){
			collection = collection+book.toString()+"\n";
		}
		return collection;
	}

	public ArrayList<Book> getBooks() {
		return books;
	}

	public String toLongString() {
		String collection = "";
		for(Book book : books){
			collection = collection+book.toLongString()+"\n";
		}
		return collection;
	}

}
