package ui;

import io.Load;
import io.Save;

import java.awt.*;
import java.awt.event.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;

import logic.*;

public class GUI {
	private static JFrame mainFrame = new JFrame("Die Magische Bibliothek");
	private static JButton generateRandomBooks = new JButton("Generieren!");
	private static JButton saveAsTxtFile = new JButton("Als Textdatei speichern!");
	private static JButton saveAsFile = new JButton("Daten speichern");
	private static JButton loadFile = new JButton("Daten ffnen");
	private static JTextField countInput = new JTextField(3);
	private Font garamond = new Font(Font.SERIF, 1, 12);
	private JList bookList = new JList();
	private JPopupMenu contextMenu = new JPopupMenu();
	private JMenuItem showDetails = new JMenuItem("Zeige Details");
	private JMenuItem saveBook = new JMenuItem("Gesamtes Buch als Textdatei speichern...");
	private JMenuItem delete = new JMenuItem("Buch lschen");
	
	public GUI(){
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainFrame.setResizable(false);
		mainFrame.setLayout(new BorderLayout());
		mainFrame.setSize(750, 500);
		updateBookListModel();
		contextMenu.add(showDetails);
		contextMenu.add(saveBook);
		contextMenu.addSeparator();
		contextMenu.add(delete);
		JScrollPane mainPane = new JScrollPane(bookList);
		mainPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		mainPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		mainPane.setToolTipText("Klicken Sie hier, um ein Kontextmen fr ein Buch anzuzeigen!");
		Box eastBox = Box.createHorizontalBox();
		JLabel newbooks = new JLabel("Anzahl neuer Bcher: ");
		eastBox.add(newbooks);
		eastBox.add(countInput);
		eastBox.add(generateRandomBooks);
		eastBox.add(saveAsTxtFile);
		eastBox.add(loadFile);
		eastBox.add(saveAsFile);
		mainFrame.add(mainPane);
		mainFrame.add(eastBox, BorderLayout.SOUTH);
		theActionListeners();
		countInput.setFont(garamond);
		generateRandomBooks.setFont(garamond);
		saveAsTxtFile.setFont(garamond);
		saveAsFile.setFont(garamond);
		loadFile.setFont(garamond);
		showDetails.setFont(garamond);
		delete.setFont(garamond);
		saveBook.setFont(garamond);
		bookList.setFont(garamond);
		newbooks.setFont(garamond);
		mainFrame.setVisible(true);
	}

	/**
	 * The File Save Dialogue, shows only .txt-Files (to overwrite,
	 * without prompt!) and saves the given string to a file
	 * with the extension .txt
	 * @param string the string to save as textfile
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void saveDialog(String string, String savename) throws FileNotFoundException, IOException{
		final JFileChooser fc = new JFileChooser();
		FileNameExtensionFilter filter = new FileNameExtensionFilter("Textdateien", "txt");
		fc.setFileFilter(filter);
		fc.setAcceptAllFileFilterUsed(false);
		int returnVal = fc.showSaveDialog(mainFrame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();
			String txt = file.getAbsolutePath();
			if(!txt.endsWith(".txt")){
				txt = txt+".txt";
			}
			Save.saveAsTxtFile(txt, string);
		}
	}
	
	public void updateBookListModel(){
		DefaultListModel model = new DefaultListModel();
		for(Book book : BookCollection.getInstance().getBooks()){
			model.addElement(book);
		}
		this.bookList.setModel(model);
	}
	
	public void generateBooks(){
		int i = 0;
		int j = 0;
		try {
			j = Integer.valueOf(countInput.getText());
		} catch (NumberFormatException e1) {
			JOptionPane.showMessageDialog(mainFrame, "Sie mssen einen gltigen Wert in das Textfeld eingeben!");
		}
		if(j>100){
			JOptionPane.showMessageDialog(mainFrame, "Aus Performancegrnden knnen nicht mehr als 100 Bcher gleichzeitig generiert werden!");
			j=100;
		}
		while(i<j){
			BookCollection.getInstance().addBook();
			i++;
		}
		countInput.setText("");
		updateBookListModel();
	}
	
	public void theActionListeners(){
		countInput.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {}

			public void keyReleased(KeyEvent e) {}

			public void keyTyped(KeyEvent e) {
				if (e.getKeyChar() == KeyEvent.VK_ENTER) {
					generateBooks();
				}
			}
		});
		showDetails.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				showDetails((Book)bookList.getSelectedValue());
			}
		});
		saveBook.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				Book book = (Book)bookList.getSelectedValue();
				try {
					saveDialog(book.toLongString(), book.getTitle());
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		});
		delete.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				Book book = (Book)bookList.getSelectedValue();
				try {
					BookCollection.getInstance().removeBook(book.getId());
				} catch (BookNotFoundException e1) {
					e1.printStackTrace();
				}
				updateBookListModel();
			}
		});
		bookList.addMouseListener(new MouseListener(){
			public void mouseEntered(MouseEvent e){}
			public void mouseExited(MouseEvent e){}
			public void mousePressed(MouseEvent e){}
			public void mouseClicked(MouseEvent e){
				contextMenu.show(bookList, e.getX(), e.getY());
			}
			public void mouseReleased(MouseEvent e){}
		});
		loadFile.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				try {
					BookCollection.setInstance((BookCollection)Load.load("mbuecher.ser"));
				} catch (FileNotFoundException e1) {
					try {
						Save.saveAsFile("mbuecher.ser", null);
					} catch (FileNotFoundException e2) {
						e2.printStackTrace();
					} catch (IOException e2) {
						e2.printStackTrace();
					}
				} catch (IOException e1) {
					try {
						Save.saveAsFile("mbuecher.ser", null);
					} catch (FileNotFoundException e2) {
						e2.printStackTrace();
					} catch (IOException e2) {
						e2.printStackTrace();
					}
				} catch (ClassNotFoundException e1) {
					e1.printStackTrace();
				}
				updateBookListModel();
			}
		});
		saveAsFile.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				try {
					Save.saveAsFile("mbuecher.ser", BookCollection.getInstance());
				} catch (FileNotFoundException e1) {
					e1.printStackTrace();
				} catch (IOException e1) {
					e1.printStackTrace();
				}
			}
		});
		generateRandomBooks.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				generateBooks();
			}
		});
		saveAsTxtFile.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				try {
					saveDialog(BookCollection.getInstance().toLongString(), "Magische Bcher");
				} catch (IOException ioe) {
					System.out.println("Fehler beim Speichern");
					ioe.printStackTrace();
				}
			}
		});
	}
	
	public void showDetails(Book book){
		final Book booky = book;
		final JFrame detailFrame = new JFrame(book.getTitle()+" "+book.getTopic().getMainTopic());
		detailFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		detailFrame.setSize(400, 400);
		detailFrame.setResizable(false);
		JTextArea deluxeInfos = new JTextArea(book.toLongStringWithoutNotices());
		deluxeInfos.setSize(400, 170);
		deluxeInfos.setLineWrap(true);
		deluxeInfos.setWrapStyleWord(true);
		deluxeInfos.setEditable(false);
		final JList notices = new JList();
		notices.setModel(updateNoticeListModel(book));
		JButton addNotice = new JButton("Notiz hinzufgen");
		addNotice.setSize(100, 30);
		JButton removeNotice = new JButton("Notiz entfernen");
		removeNotice.setSize(100, 30);
		JButton close = new JButton("Schlieen");
		removeNotice.setSize(30, 30);
		JPanel buttonPanel = new JPanel();
		buttonPanel.setSize(400, 60);
		buttonPanel.add(addNotice);
		buttonPanel.add(removeNotice);
		buttonPanel.add(close);
		JScrollPane scrollList = new JScrollPane(notices);
		scrollList.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollList.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollList.setMaximumSize(new Dimension (400, 170));
		Box contentBox = Box.createVerticalBox();
		contentBox.setSize(400, 400);
		contentBox.add(deluxeInfos);
		JLabel lblnotices = new JLabel("Notizen:");
		contentBox.add(lblnotices);
		contentBox.add(scrollList);
		contentBox.add(buttonPanel);
		detailFrame.add(contentBox);
		deluxeInfos.setFont(garamond);
		addNotice.setFont(garamond);
		removeNotice.setFont(garamond);
		close.setFont(garamond);
		lblnotices.setFont(garamond);
		detailFrame.setVisible(true);
		addNotice.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				String newNotice = JOptionPane.showInputDialog(detailFrame, "Bitte die Notiz eingeben", "Notizeingabe", JOptionPane.QUESTION_MESSAGE);
				try {
					BookCollection.getInstance().getBook(booky.getId()).addNotice(newNotice);
				} catch (BookNotFoundException e1) {
					e1.printStackTrace();
				}
				notices.setModel(updateNoticeListModel(booky));
			}
		});
		removeNotice.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				try {
					if (BookCollection.getInstance().getBook(booky.getId()).removeNotice(notices.getSelectedValue().toString())){
						JOptionPane.showMessageDialog(detailFrame, "Notiz erfolgreich gelscht!", "Notiz entfernen", JOptionPane.INFORMATION_MESSAGE);
						notices.setModel(updateNoticeListModel(booky));
					}
					else{
						JOptionPane.showMessageDialog(detailFrame, "Notiz konnte nicht entfernt werden!", "Notiz entfernen", JOptionPane.ERROR_MESSAGE);
						notices.setModel(updateNoticeListModel(booky));
					}
				} catch (BookNotFoundException e1) {
					e1.printStackTrace();
				} catch (NullPointerException e2){
					JOptionPane.showMessageDialog(detailFrame, "Bitte eine bestehende Notiz auswhlen!!", "Notiz entfernen", JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		close.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				detailFrame.dispose();
			}
		});
	}
	
	public DefaultListModel updateNoticeListModel(Book book){
		DefaultListModel model = new DefaultListModel();
		if(book.getNotices() != null){
			for(String string : book.getNotices()){
				model.addElement(string);
			}
		}
		
		return model;
	}
	
	public static void main(String[] args) {
		new GUI();
	}

}
